==> create-fly-1.21.8-6.0.8-2-sources.jar: com/zurrtum/create/content/logistics/item/filter/attribute/attributes/FluidContentsAttribute.java <==
package com.zurrtum.create.content.logistics.item.filter.attribute.attributes;

import com.mojang.serialization.MapCodec;
import com.zurrtum.create.AllItemAttributeTypes;
import com.zurrtum.create.catnip.codecs.stream.CatnipStreamCodecBuilders;
import com.zurrtum.create.catnip.codecs.stream.CatnipStreamCodecs;
import com.zurrtum.create.content.logistics.item.filter.attribute.ItemAttribute;
import com.zurrtum.create.content.logistics.item.filter.attribute.ItemAttributeType;
import com.zurrtum.create.foundation.fluid.FluidHelper;
import com.zurrtum.create.infrastructure.fluids.FluidItemInventory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;
import net.minecraft.class_156;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2246;
import net.minecraft.class_2248;
import net.minecraft.class_2561;
import net.minecraft.class_3611;
import net.minecraft.class_3612;
import net.minecraft.class_7923;
import net.minecraft.class_9129;
import net.minecraft.class_9139;

public record FluidContentsAttribute(@Nullable class_3611 fluid) implements ItemAttribute {
    public static final MapCodec<FluidContentsAttribute> CODEC = class_7923.field_41173.method_39673()
        .xmap(FluidContentsAttribute::new, FluidContentsAttribute::fluid).fieldOf("value");

    public static final class_9139<class_9129, FluidContentsAttribute> PACKET_CODEC = CatnipStreamCodecBuilders.nullable(CatnipStreamCodecs.FLUID)
        .method_56432(FluidContentsAttribute::new, FluidContentsAttribute::fluid);

    @Override
    public boolean appliesTo(class_1799 itemStack, class_1937 level) {
        try (FluidItemInventory capability = FluidHelper.getFluidInventory(itemStack)) {
            if (capability != null) {
                for (int i = 0, size = capability.size(); i < size; i++) {
                    if (capability.getStack(i).getFluid() == fluid) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    @Override
    public String getTranslationKey() {
        return "has_fluid";
    }

    @Override
    public Object[] getTranslationParameters() {
        Object parameter = "";
        if (fluid != null) {
            class_2248 block = fluid.method_15785().method_15759().method_26204();
            if (fluid != class_3612.field_15906 && block == class_2246.field_10124) {
                parameter = class_2561.method_43471(class_156.method_646("fluid", class_7923.field_41173.method_10221(fluid)));
            } else {
                parameter = block.method_9518();
            }
        }
        return new Object[]{parameter};
    }

    @Override
    public ItemAttributeType getType() {
        return AllItemAttributeTypes.HAS_FLUID;
    }

    public static class Type implements ItemAttributeType {
        @Override
        public @NotNull ItemAttribute createAttribute() {
            return new FluidContentsAttribute(null);
        }

        @Override
        public List<ItemAttribute> getAllAttributes(class_1799 stack, class_1937 level) {
            List<ItemAttribute> list = new ArrayList<>();

            try (FluidItemInventory capability = FluidHelper.getFluidInventory(stack)) {
                if (capability != null) {
                    for (int i = 0, size = capability.size(); i < size; i++) {
                        list.add(new FluidContentsAttribute(capability.getStack(i).getFluid()));
                    }
                }
            }

            return list;
        }

        @Override
        public MapCodec<? extends ItemAttribute> codec() {
            return CODEC;
        }

        @Override
        public class_9139<? super class_9129, ? extends ItemAttribute> packetCodec() {
            return PACKET_CODEC;
        }
    }
}

==> create-fly-1.21.11-6.0.8-2-sources.jar: com/zurrtum/create/content/logistics/item/filter/attribute/attributes/FluidContentsAttribute.java <==
package com.zurrtum.create.content.logistics.item.filter.attribute.attributes;

import com.mojang.serialization.MapCodec;
import com.zurrtum.create.AllItemAttributeTypes;
import com.zurrtum.create.catnip.codecs.stream.CatnipStreamCodecBuilders;
import com.zurrtum.create.catnip.codecs.stream.CatnipStreamCodecs;
import com.zurrtum.create.content.logistics.item.filter.attribute.ItemAttribute;
import com.zurrtum.create.content.logistics.item.filter.attribute.ItemAttributeType;
import com.zurrtum.create.foundation.fluid.FluidHelper;
import com.zurrtum.create.infrastructure.fluids.FluidItemInventory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;
import net.minecraft.class_156;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2246;
import net.minecraft.class_2248;
import net.minecraft.class_2561;
import net.minecraft.class_3611;
import net.minecraft.class_3612;
import net.minecraft.class_7923;
import net.minecraft.class_9129;
import net.minecraft.class_9139;

public record FluidContentsAttribute(@Nullable class_3611 fluid) implements ItemAttribute {
    public static final MapCodec<FluidContentsAttribute> CODEC = class_7923.field_41173.method_39673()
        .xmap(FluidContentsAttribute::new, FluidContentsAttribute::fluid).fieldOf("value");

    public static final class_9139<class_9129, FluidContentsAttribute> PACKET_CODEC = CatnipStreamCodecBuilders.nullable(
        CatnipStreamCodecs.FLUID).method_56432(FluidContentsAttribute::new, FluidContentsAttribute::fluid);

    @Override
    public boolean appliesTo(class_1799 itemStack, class_1937 level) {
        try (FluidItemInventory capability = FluidHelper.getFluidInventory(itemStack)) {
            if (capability != null) {
                for (int i = 0, size = capability.size(); i < size; i++) {
                    if (capability.getStack(i).getFluid() == fluid) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    @Override
    public String getTranslationKey() {
        return "has_fluid";
    }

    @Override
    public Object[] getTranslationParameters() {
        Object parameter = "";
        if (fluid != null) {
            class_2248 block = fluid.method_15785().method_15759().method_26204();
            if (fluid != class_3612.field_15906 && block == class_2246.field_10124) {
                parameter = class_2561.method_43471(class_156.method_646("block", class_7923.field_41173.method_10221(fluid)));
            } else {
                parameter = block.method_9518();
            }
        }
        return new Object[]{parameter};
    }

    @Override
    public ItemAttributeType getType() {
        return AllItemAttributeTypes.HAS_FLUID;
    }

    public static class Type implements ItemAttributeType {
        @Override
        public @NotNull ItemAttribute createAttribute() {
            return new FluidContentsAttribute(null);
        }

        @Override
        public List<ItemAttribute> getAllAttributes(class_1799 stack, class_1937 level) {
            List<ItemAttribute> list = new ArrayList<>();

            try (FluidItemInventory capability = FluidHelper.getFluidInventory(stack)) {
                if (capability != null) {
                    for (int i = 0, size = capability.size(); i < size; i++) {
                        list.add(new FluidContentsAttribute(capability.getStack(i).getFluid()));
                    }
                }
            }

            return list;
        }

        @Override
        public MapCodec<? extends ItemAttribute> codec() {
            return CODEC;
        }

        @Override
        public class_9139<? super class_9129, ? extends ItemAttribute> packetCodec() {
            return PACKET_CODEC;
        }
    }
}
